/* This class converts what's presumably a ISO8859-1 String containg UTF-8
 * bytes to a String with UTF-8. Yes, midp 1.0 should have supported utf-8. 
 *
 * Vidar Holen, for Virca
 * http://www.vidarholen.net
 *
 * Released to the public domain.
 */

public class UTF8 implements Codepage {
    public String getName() { return "UTF-8"; }
    public void init() {}

    //Attempt ugly conversion. If it fails (ie utf-8 isn't proper), bail out
    //with original string (so we get iso8859-1 instead. Might have made this
    //choosable). Only bothers with 3-byte codes. This should have used byte[],
    //bah.
    public String serverToUnicode(String s) {
        StringBuffer sb=new StringBuffer();
        char[] a=s.toCharArray();
        char t;
        int cnt;
        for(int i=0; i<a.length; i++) {
            if(a[i]<0x80) {
                sb.append(a[i]);
            } else if(a[i]>0xEF) {
                return s; //with 16-bit chars, we don't bother with more
            } else {
                t=a[i]; 
                if((t|0xE0)==t) cnt=2; 
                else if((t|0xC0)==t) cnt=1;
                else return s;
                t=(char)(t&(((1<<(6-cnt))-1)));
                for(int j=0; j<cnt; j++) {
                    i++;
                    if(i==a.length || a[i]>0xBF) return s;
                    int k=t;
                    t=(char)((t<<6)|(a[i]&0x3f));
                }
                sb.append(t);
            }
        }
        return sb.toString();
    }
    public byte[] unicodeToServer(String s) {
        byte[] b=new byte[strlen(s)];
        char[] a=s.toCharArray();
        int j=0;
        for(int i=0; i<a.length; i++) {
            if(a[i]<0x80) {
                b[j]=(byte)(a[i]);
                j+=1;
            } else if(a[i]<0x800) {
                b[j]=(byte)(0xC0 | (a[i]>>6));
                b[j+1]=(byte)(0x80 | (a[i]&0x3F));
                j+=2;
            } else {
                b[j]=(byte)(0xE0 | (a[i]>>12));
                b[j+1]=(byte)(0x80 | ((a[i]>>6)&0x3F));
                b[j+2]=(byte)(0x80 | (a[i]&0x3F));
                j+=3;
            }
        }

        return b;
    }
    public byte[] serverToServer(String s) {
        return s.getBytes();
    }

    //Find length in bytes of a string, akin to strlen vs wcslen
    public static int strlen(String s) {
        int n=0;
        char[] a=s.toCharArray();
        for(int i=0; i<a.length; i++) {
            if(a[i]<0x80) n++;
            else if(a[i]<0x800) n+=2;
            else n+=3;
        }
        return n;
    }
}

